/**
 * Interface of the server (auto-generated code)
 */
import {
	HTTPMimeType,
	HTTPRequestModel,
	RESTCallbacks,
	RESTConfig,
	RESTRequestJson,
	RESTRequestVoid,
} from "../rest-tools";

import { z as zod } from "zod"
import {
	Artist,
	ArtistWrite,
	Long,
	ObjectId,
	ZodArtist,
	isArtist,
} from "../model";

export namespace ArtistResource {

	export function get({
			restConfig,
			params,
		}: {
		restConfig: RESTConfig,
		params: {
			id: Long,
		},
	}): Promise<Artist> {
		return RESTRequestJson({
			restModel: {
				endPoint: "/artist/{id}",
				requestType: HTTPRequestModel.GET,
				accept: HTTPMimeType.JSON,
			},
			restConfig,
			params,
		}, isArtist);
	};

	export const ZodGetsTypeReturn = zod.array(ZodArtist);
	export type GetsTypeReturn = zod.infer<typeof ZodGetsTypeReturn>;
	
	export function isGetsTypeReturn(data: any): data is GetsTypeReturn {
		try {
			ZodGetsTypeReturn.parse(data);
			return true;
		} catch (e: any) {
			console.log(`Fail to parse data type='ZodGetsTypeReturn' error=${e}`);
			return false;
		}
	}

	export function gets({
			restConfig,
		}: {
		restConfig: RESTConfig,
	}): Promise<GetsTypeReturn> {
		return RESTRequestJson({
			restModel: {
				endPoint: "/artist/",
				requestType: HTTPRequestModel.GET,
				accept: HTTPMimeType.JSON,
			},
			restConfig,
		}, isGetsTypeReturn);
	};
	export function patch({
			restConfig,
			params,
			data,
		}: {
		restConfig: RESTConfig,
		params: {
			id: Long,
		},
		data: ArtistWrite,
	}): Promise<Artist> {
		return RESTRequestJson({
			restModel: {
				endPoint: "/artist/{id}",
				requestType: HTTPRequestModel.PATCH,
				contentType: HTTPMimeType.JSON,
				accept: HTTPMimeType.JSON,
			},
			restConfig,
			params,
			data,
		}, isArtist);
	};
	export function post({
			restConfig,
			data,
		}: {
		restConfig: RESTConfig,
		data: ArtistWrite,
	}): Promise<Artist> {
		return RESTRequestJson({
			restModel: {
				endPoint: "/artist/",
				requestType: HTTPRequestModel.POST,
				contentType: HTTPMimeType.JSON,
				accept: HTTPMimeType.JSON,
			},
			restConfig,
			data,
		}, isArtist);
	};
	export function remove({
			restConfig,
			params,
		}: {
		restConfig: RESTConfig,
		params: {
			id: Long,
		},
	}): Promise<void> {
		return RESTRequestVoid({
			restModel: {
				endPoint: "/artist/{id}",
				requestType: HTTPRequestModel.DELETE,
				contentType: HTTPMimeType.TEXT_PLAIN,
			},
			restConfig,
			params,
		});
	};
	export function removeCover({
			restConfig,
			params,
		}: {
		restConfig: RESTConfig,
		params: {
			coverId: ObjectId,
			id: Long,
		},
	}): Promise<Artist> {
		return RESTRequestJson({
			restModel: {
				endPoint: "/artist/{id}/cover/{coverId}",
				requestType: HTTPRequestModel.DELETE,
				contentType: HTTPMimeType.TEXT_PLAIN,
				accept: HTTPMimeType.JSON,
			},
			restConfig,
			params,
		}, isArtist);
	};
	export function uploadCover({
			restConfig,
			params,
			data,
			callbacks,
		}: {
		restConfig: RESTConfig,
		params: {
			id: Long,
		},
		data: {
			file?: File,
			uri?: string,
		},
		callbacks?: RESTCallbacks,
	}): Promise<Artist> {
		return RESTRequestJson({
			restModel: {
				endPoint: "/artist/{id}/cover",
				requestType: HTTPRequestModel.POST,
				contentType: HTTPMimeType.MULTIPART,
				accept: HTTPMimeType.JSON,
			},
			restConfig,
			params,
			data,
			callbacks,
		}, isArtist);
	};
}
