/**
 * Interface of the server (auto-generated code)
 */
import { z as zod } from "zod";

import {ZodObjectId} from "./object-id";
import {ZodLocalDate} from "./local-date";
import {ZodGenericDataSoftDelete, ZodGenericDataSoftDeleteWrite } from "./generic-data-soft-delete";

export const ZodAlbum = ZodGenericDataSoftDelete.extend({
	name: zod.string().max(256).optional(),
	description: zod.string().optional(),
	/**
	 * List of Id of the specific covers
	 */
	covers: zod.array(ZodObjectId).optional(),
	publication: ZodLocalDate.optional(),

});

export type Album = zod.infer<typeof ZodAlbum>;

export function isAlbum(data: any): data is Album {
	try {
		ZodAlbum.parse(data);
		return true;
	} catch (e: any) {
		console.log(`Fail to parse data type='ZodAlbum' error=${e}`);
		return false;
	}
}
export const ZodAlbumWrite = ZodGenericDataSoftDeleteWrite.extend({
	name: zod.string().max(256).nullable().optional(),
	description: zod.string().nullable().optional(),
	/**
	 * List of Id of the specific covers
	 */
	covers: zod.array(ZodObjectId).nullable().optional(),
	publication: ZodLocalDate.nullable().optional(),

});

export type AlbumWrite = zod.infer<typeof ZodAlbumWrite>;

export function isAlbumWrite(data: any): data is AlbumWrite {
	try {
		ZodAlbumWrite.parse(data);
		return true;
	} catch (e: any) {
		console.log(`Fail to parse data type='ZodAlbumWrite' error=${e}`);
		return false;
	}
}
